@@ -13,7 +13,7 @@
 ##############################################################################
 """Interface object implementation
 
-$Id: interface.py 27081 2004-08-12 19:56:31Z srichter $
+$Id: interface.py 82877 2008-01-14 20:58:02Z witsch $
 """
 
 from __future__ import generators
@@ -25,19 +25,28 @@ from types import FunctionType
 from ro import ro
 from zope.interface.exceptions import Invalid
 
+
 CO_VARARGS = 4
 CO_VARKEYWORDS = 8
 TAGGED_DATA = '__interface_tagged_values__'
 
+_decorator_non_return = object()
+
 def invariant(call):
     f_locals = sys._getframe(1).f_locals
-    tags = f_locals.get(TAGGED_DATA)
-    if tags is None:
-        tags = f_locals[TAGGED_DATA] = {}
-    invariants = tags.get('invariants')
-    if invariants is None:
-        invariants = tags['invariants'] = []
+    tags = f_locals.setdefault(TAGGED_DATA, {})
+    invariants = tags.setdefault('invariants', [])
     invariants.append(call)
+    return _decorator_non_return
+
+
+def taggedValue(key, value):
+    """Attaches a tagged value to an interface at definition time."""
+    f_locals = sys._getframe(1).f_locals
+    tagged_values = f_locals.setdefault(TAGGED_DATA, {})
+    tagged_values[key] = value
+    return _decorator_non_return
+
 
 class Element(object):
 
@@ -107,13 +116,14 @@ class SpecificationBasePy(object):
           >>> directlyProvides(C, I1)
           >>> I1.providedBy(C)
           True
-        
+
         """
         spec = providedBy(ob)
         return self in spec._implied
 
     def implementedBy(self, cls):
-        """Do instances of the given class implement the interface?"""
+        """Test whether the specification is implemented by a class or factory.
+        Raise TypeError if argument is neither a class nor a callable."""
         spec = implementedBy(cls)
         return self in spec._implied
 
@@ -134,7 +144,7 @@ class SpecificationBasePy(object):
           ...
           >>> spec = Declaration()
           >>> int(spec.extends(Interface))
-          0
+          1
           >>> spec = Declaration(I2)
           >>> int(spec.extends(Interface))
           1
@@ -150,12 +160,55 @@ class SpecificationBasePy(object):
         """
         return interface in self._implied
 
+    __call__ = isOrExtends
+
 SpecificationBase = SpecificationBasePy
 
+_marker = object()
+class InterfaceBasePy(object):
+    """Base class that wants to be replaced with a C base :)
+    """
+
+    def __call__(self, obj, alternate=_marker):
+        """Adapt an object to the interface
+        """
+        conform = getattr(obj, '__conform__', None)
+        if conform is not None:
+            adapter = self._call_conform(conform)
+            if adapter is not None:
+                return adapter
+
+        adapter = self.__adapt__(obj)
+
+        if adapter is not None:
+            return adapter
+        elif alternate is not _marker:
+            return alternate
+        else:
+            raise TypeError("Could not adapt", obj, self)
+
+    def __adapt__(self, obj):
+        """Adapt an object to the reciever
+        """
+        if self.providedBy(obj):
+            return obj
+
+        for hook in adapter_hooks:
+            adapter = hook(self, obj)
+            if adapter is not None:
+                return adapter
+    
+InterfaceBase = InterfaceBasePy
+
+adapter_hooks = []
+
 try:
-    from _zope_interface_coptimizations import SpecificationBase
+    import _zope_interface_coptimizations
 except ImportError:
     pass
+else:
+    from _zope_interface_coptimizations import SpecificationBase
+    from _zope_interface_coptimizations import InterfaceBase, adapter_hooks
 
 class Specification(SpecificationBase):
     """Specifications
@@ -195,61 +248,49 @@ class Specification(SpecificationBase):
 
     >>> I3.extends(I1)
     0
-        
+
     """
 
     # Copy some base class methods for speed
     isOrExtends = SpecificationBase.isOrExtends
     providedBy = SpecificationBase.providedBy
 
-    #########################################################################
-    # XXX Backward Compat
-    def isImplementedByInstancesOf(self, cls):
-        warnings.warn(
-            "isImplementedByInstancesOf has been renamed to implementedBy",
-            DeprecationWarning, stacklevel=2,
-            )
-        return self.implementedBy(cls)
-
-    def isImplementedBy(self, ob):
-        warnings.warn(
-            "isImplementedBy has been renamed to providedBy",
-            DeprecationWarning, stacklevel=2,
-            )
-        return self.providedBy(ob)
-    #
-    #########################################################################
-
     def __init__(self, bases=()):
         self._implied = {}
         self.dependents = weakref.WeakKeyDictionary()
         self.__bases__ = tuple(bases)
 
     def subscribe(self, dependent):
-        self.dependents[dependent] = 1
+        self.dependents[dependent] = self.dependents.get(dependent, 0) + 1
 
     def unsubscribe(self, dependent):
-        del self.dependents[dependent]
+        n = self.dependents.get(dependent, 0) - 1
+        if not n:
+            del self.dependents[dependent]
+        elif n > 0:
+            self.dependents[dependent] = n
+        else:
+            raise KeyError(dependent)
 
     def __setBases(self, bases):
         # Register ourselves as a dependent of our old bases
         for b in self.__bases__:
             b.unsubscribe(self)
-        
+
         # Register ourselves as a dependent of our bases
         self.__dict__['__bases__'] = bases
         for b in bases:
             b.subscribe(self)
-        
-        self.changed()
+
+        self.changed(self)
 
     __bases__ = property(
-        
+
         lambda self: self.__dict__.get('__bases__', ()),
         __setBases,
         )
 
-    def changed(self):
+    def changed(self, originally_changed):
         """We, or something we depend on, have changed
         """
 
@@ -257,10 +298,17 @@ class Specification(SpecificationBase):
         implied.clear()
 
         ancestors = ro(self)
+
+        try:
+            if Interface not in ancestors:
+                ancestors.append(Interface)
+        except NameError:
+            pass # defining Interface itself
+
         self.__sro__ = tuple(ancestors)
         self.__iro__ = tuple([ancestor for ancestor in ancestors
                               if isinstance(ancestor, InterfaceClass)
-                             ])
+                              ])
 
         for ancestor in ancestors:
             # We directly imply our ancestors:
@@ -268,7 +316,7 @@ class Specification(SpecificationBase):
 
         # Now, advise our dependents of change:
         for dependent in self.dependents.keys():
-            dependent.changed()
+            dependent.changed(originally_changed)
 
 
     def interfaces(self):
@@ -303,7 +351,7 @@ class Specification(SpecificationBase):
                 if interface not in seen:
                     seen[interface] = 1
                     yield interface
-        
+
 
     def extends(self, interface, strict=True):
         """Does the specification extend the given interface?
@@ -325,7 +373,7 @@ class Specification(SpecificationBase):
           ...
           >>> spec = Declaration()
           >>> int(spec.extends(Interface))
-          0
+          1
           >>> spec = Declaration(I2)
           >>> int(spec.extends(Interface))
           1
@@ -351,13 +399,29 @@ class Specification(SpecificationBase):
                 )
 
     def weakref(self, callback=None):
-        if callback is None:
-            return weakref.ref(self)
-        else:
-            return weakref.ref(self, callback)
-        
+        return weakref.ref(self, callback)
 
-class InterfaceClass(Element, Specification):
+    def get(self, name, default=None):
+        """Query for an attribute description
+        """
+        try:
+            attrs = self._v_attrs
+        except AttributeError:
+            attrs = self._v_attrs = {}
+        attr = attrs.get(name)
+        if attr is None:
+            for iface in self.__iro__:
+                attr = iface.direct(name)
+                if attr is not None:
+                    attrs[name] = attr
+                    break
+
+        if attr is None:
+            return default
+        else:
+            return attr
+
+class InterfaceClass(Element, InterfaceBase, Specification):
     """Prototype (scarecrow) Interfaces Implementation."""
 
     # We can't say this yet because we don't have enough
@@ -368,15 +432,14 @@ class InterfaceClass(Element, Specification):
     def __init__(self, name, bases=(), attrs=None, __doc__=None,
                  __module__=None):
 
+        if attrs is None:
+            attrs = {}
+
         if __module__ is None:
-            if (attrs is not None and
-                ('__module__' in attrs) and
-                isinstance(attrs['__module__'], str)
-                ):
-                __module__ = attrs['__module__']
+            __module__ = attrs.get('__module__')
+            if isinstance(__module__, str):
                 del attrs['__module__']
             else:
-
                 try:
                     # Figure out what module defined the interface.
                     # This is how cPython figures out the module of
@@ -387,9 +450,6 @@ class InterfaceClass(Element, Specification):
 
         self.__module__ = __module__
 
-        if attrs is None:
-            attrs = {}
-
         d = attrs.get('__doc__')
         if d is not None:
             if not isinstance(d, Attribute):
@@ -402,30 +462,30 @@ class InterfaceClass(Element, Specification):
 
         Element.__init__(self, name, __doc__)
 
-        if attrs.has_key(TAGGED_DATA):
-            tagged_data = attrs[TAGGED_DATA]
-            del attrs[TAGGED_DATA]
-        else:
-            tagged_data = None
+        tagged_data = attrs.pop(TAGGED_DATA, None)
         if tagged_data is not None:
             for key, val in tagged_data.items():
                 self.setTaggedValue(key, val)
 
-        for b in bases:
-            if not isinstance(b, InterfaceClass):
-                raise TypeError, 'Expected base interfaces'
+        for base in bases:
+            if not isinstance(base, InterfaceClass):
+                raise TypeError('Expected base interfaces')
 
         Specification.__init__(self, bases)
 
-        for k, v in attrs.items():
-            if isinstance(v, Attribute):
-                v.interface = name
-                if not v.__name__:
-                    v.__name__ = k
-            elif isinstance(v, FunctionType):
-                attrs[k] = fromFunction(v, name, name=k)
+        # Make sure that all recorded attributes (and methods) are of type
+        # `Attribute` and `Method`
+        for name, attr in attrs.items():
+            if isinstance(attr, Attribute):
+                attr.interface = self
+                if not attr.__name__:
+                    attr.__name__ = name
+            elif isinstance(attr, FunctionType):
+                attrs[name] = fromFunction(attr, self, name=name)
+            elif attr is _decorator_non_return:
+                del attrs[name]
             else:
-                raise InvalidInterface("Concrete attribute, %s" % k)
+                raise InvalidInterface("Concrete attribute, " + name)
 
         self.__attrs = attrs
 
@@ -439,7 +499,7 @@ class InterfaceClass(Element, Specification):
           >>> from zope.interface import Interface
           >>> class I1(Interface): pass
           ...
-          >>> 
+          >>>
           >>> i = I1.interfaces()
           >>> i.next().getName()
           'I1'
@@ -448,28 +508,23 @@ class InterfaceClass(Element, Specification):
         """
         yield self
 
-
-
     def getBases(self):
         return self.__bases__
 
     def isEqualOrExtendedBy(self, other):
         """Same interface or extends?"""
-        if self == other:
-            return True
-        return other.extends(self)
+        return self == other or other.extends(self)
 
     def names(self, all=False):
         """Return the attribute names defined by the interface."""
         if not all:
             return self.__attrs.keys()
 
-        r = {}
-        for name in self.__attrs.keys():
-            r[name] = 1
+        r = self.__attrs.copy()
+
         for base in self.__bases__:
-            for name in base.names(all):
-                r[name] = 1
+            r.update(dict.fromkeys(base.names(all)))
+
         return r.keys()
 
     def __iter__(self):
@@ -481,42 +536,31 @@ class InterfaceClass(Element, Specification):
             return self.__attrs.items()
 
         r = {}
-        for name, d in self.__attrs.items():
-            r[name] = d
+        for base in self.__bases__[::-1]:
+            r.update(dict(base.namesAndDescriptions(all)))
 
-        for base in self.__bases__:
-            for name, d in base.namesAndDescriptions(all):
-                if name not in r:
-                    r[name] = d
+        r.update(self.__attrs)
 
         return r.items()
 
     def getDescriptionFor(self, name):
         """Return the attribute description for the given name."""
-        r = self.queryDescriptionFor(name)
+        r = self.get(name)
         if r is not None:
             return r
 
-        raise KeyError, name
+        raise KeyError(name)
 
     __getitem__ = getDescriptionFor
 
     def __contains__(self, name):
-        return self.queryDescriptionFor(name) is not None
+        return self.get(name) is not None
+
+    def direct(self, name):
+        return self.__attrs.get(name)
 
     def queryDescriptionFor(self, name, default=None):
-        """Return the attribute description for the given name."""
-        r = self.__attrs.get(name, self)
-        if r is not self:
-            return r
-        for base in self.__bases__:
-            r = base.queryDescriptionFor(name, self)
-            if r is not self:
-                return r
-
-        return default
-
-    get = queryDescriptionFor
+        return self.get(name, default)
 
     def deferred(self):
         """Return a defered class corresponding to the interface."""
@@ -548,7 +592,6 @@ class InterfaceClass(Element, Specification):
             except Invalid:
                 if errors is None:
                     raise
-                pass
         if errors:
             raise Invalid(errors)
 
@@ -562,188 +605,39 @@ class InterfaceClass(Element, Specification):
             if isinstance(v, Method) and not (k in dict):
                 dict[k]=v
 
-        for b in self.__bases__: b.__d(dict)
+        for b in self.__bases__:
+            b.__d(dict)
 
     def __repr__(self):
-        r = getattr(self, '_v_repr', self)
-        if r is self:
+        try:
+            return self._v_repr
+        except AttributeError:
             name = self.__name__
             m = self.__module__
             if m:
                 name = '%s.%s' % (m, name)
             r = "<%s %s>" % (self.__class__.__name__, name)
             self._v_repr = r
-        return r
+            return r
 
-    def __call__():
-        # TRICK! Create the call method
-        #
-        # An embedded function is used to allow an optional argument to
-        # __call__ without resorting to a global marker.
-        #
-        # The evility of this trick is a reflection of the underlying
-        # evility of "optional" arguments, arguments whos presense or
-        # absense changes the behavior of the methos.
-        # 
-        # I think the evil is necessary, and perhaps desireable to
-        # provide some consistencey with the PEP 246 adapt method.
+    def _call_conform(self, conform):
+        try:
+            return conform(self)
+        except TypeError:
+            # We got a TypeError. It might be an error raised by
+            # the __conform__ implementation, or *we* may have
+            # made the TypeError by calling an unbound method
+            # (object is a class).  In the later case, we behave
+            # as though there is no __conform__ method. We can
+            # detect this case by checking whether there is more
+            # than one traceback object in the traceback chain:
+            if sys.exc_info()[2].tb_next is not None:
+                # There is more than one entry in the chain, so
+                # reraise the error:
+                raise
+            # This clever trick is from Phillip Eby
 
-        marker = object()
-        
-        def __call__(self, obj, alternate=marker):
-            """Adapt an object to the interface
-
-               The sematics based on those of the PEP 246 adapt function.
-
-               If an object cannot be adapted, then a TypeError is raised::
-
-                 >>> import zope.interface
-                 >>> class I(zope.interface.Interface):
-                 ...     pass
-
-                 >>> I(0)
-                 Traceback (most recent call last):
-                 ...
-                 TypeError: ('Could not adapt', 0, """ \
-                      """<InterfaceClass zope.interface.interface.I>)
-
-               unless an alternate value is provided as a second
-               positional argument::
-
-                 >>> I(0, 'bob')
-                 'bob'
-
-               If an object already implements the interface, then it will be
-               returned::
-
-                 >>> class C(object):
-                 ...     zope.interface.implements(I)
-
-                 >>> obj = C()
-                 >>> I(obj) is obj
-                 True
-
-               If an object implements __conform__, then it will be used::
-
-                 >>> class C(object):
-                 ...     zope.interface.implements(I)
-                 ...     def __conform__(self, proto):
-                 ...          return 0
-
-                 >>> I(C())
-                 0
-
-               Adapter hooks (see __adapt__) will also be used, if present:
-
-                 >>> from zope.interface.interface import adapter_hooks
-                 >>> def adapt_0_to_42(iface, obj):
-                 ...     if obj == 0:
-                 ...         return 42
-
-                 >>> adapter_hooks.append(adapt_0_to_42)
-                 >>> I(0)
-                 42
-
-                 >>> adapter_hooks.remove(adapt_0_to_42)
-                 >>> I(0)
-                 Traceback (most recent call last):
-                 ...
-                 TypeError: ('Could not adapt', 0, """ \
-                      """<InterfaceClass zope.interface.interface.I>)
-
-            """
-            conform = getattr(obj, '__conform__', None)
-            if conform is not None:
-                try:
-                    adapter = conform(self)
-                except TypeError:
-                    # We got a TypeError. It might be an error raised by
-                    # the __conform__ implementation, or *we* may have
-                    # made the TypeError by calling an unbound method
-                    # (object is a class).  In the later case, we behave
-                    # as though there is no __conform__ method. We can
-                    # detect this case by checking whether there is more
-                    # than one traceback object in the traceback chain:
-                    if sys.exc_info()[2].tb_next is not None:
-                        # There is more than one entry in the chain, so
-                        # reraise the error:
-                        raise
-                    # This clever trick is from Phillip Eby
-                else:
-                    if adapter is not None:
-                        return adapter
-
-            adapter = self.__adapt__(obj)
-
-            if adapter is None:
-                if alternate is not marker:
-                    return alternate
-                
-                raise TypeError("Could not adapt", obj, self)
-
-            return adapter
-
-        return __call__
-
-    __call__ = __call__() # TRICK! Make the *real* __call__ method
-
-    def __adapt__(self, obj):
-        """Adapt an object to the reciever
-
-           This method is normally not called directly. It is called by
-           the PEP 246 adapt framework and by the interface __call__
-           operator. 
-
-           The adapt method is responsible for adapting an object to
-           the reciever.
-
-           The default version returns None::
-
-             >>> import zope.interface
-             >>> class I(zope.interface.Interface):
-             ...     pass
-
-             >>> I.__adapt__(0)
-
-           unless the object given provides the interface::
-
-             >>> class C(object):
-             ...     zope.interface.implements(I)
-
-             >>> obj = C()
-             >>> I.__adapt__(obj) is obj
-             True
-
-           Adapter hooks can be provided (or removed) to provide custom
-           adaptation. We'll install a silly hook that adapts 0 to 42.
-           We install a hook by simply adding it to the adapter_hooks
-           list::
-
-             >>> from zope.interface.interface import adapter_hooks
-             >>> def adapt_0_to_42(iface, obj):
-             ...     if obj == 0:
-             ...         return 42
-
-             >>> adapter_hooks.append(adapt_0_to_42)
-             >>> I.__adapt__(0)
-             42
-
-           Hooks must either return an adapter, or None if no adapter can
-           be found.
-
-           Hooks can be uninstalled by removing them from the list::
-
-             >>> adapter_hooks.remove(adapt_0_to_42)
-             >>> I.__adapt__(0)
-
-           """
-        if self.providedBy(obj):
-            return obj
-
-        for hook in adapter_hooks:
-            adapter = hook(self, obj)
-            if adapter is not None:
-                return adapter
+        return None
 
     def __reduce__(self):
         return self.__name__
@@ -778,7 +672,6 @@ class InterfaceClass(Element, Specification):
         if o2 is None:
             return -1
 
-
         n1 = (getattr(o1, '__name__', ''),
               getattr(getattr(o1,  '__module__', None), '__name__', ''))
         n2 = (getattr(o2, '__name__', ''),
@@ -797,8 +690,6 @@ class InterfaceClass(Element, Specification):
         return c > 0
 
 
-adapter_hooks = []
-
 Interface = InterfaceClass("Interface", __module__ = 'zope.interface')
 
 class Attribute(Element):
@@ -808,7 +699,10 @@ class Attribute(Element):
     # We can't say this yet because we don't have enough
     # infrastructure in place.
     #
-    #__implemented__ = IAttribute
+    # implements(IAttribute)
+
+    interface = None
+
 
 class Method(Attribute):
     """Method interfaces
@@ -820,9 +714,7 @@ class Method(Attribute):
     # We can't say this yet because we don't have enough
     # infrastructure in place.
     #
-    #__implemented__ = IMethod
-
-    interface=''
+    # implements(IMethod)
 
     def __call__(self, *args, **kw):
         raise BrokenImplementation(self.interface, self.__name__)
@@ -836,66 +728,67 @@ class Method(Attribute):
                 }
 
     def getSignatureString(self):
-        sig = "("
+        sig = []
         for v in self.positional:
-            sig = sig + v
+            sig.append(v)
             if v in self.optional.keys():
-                sig = sig + "=%s" % `self.optional[v]`
-            sig = sig + ", "
+                sig[-1] += "=" + `self.optional[v]`
         if self.varargs:
-            sig = sig + ("*%s, " % self.varargs)
+            sig.append("*" + self.varargs)
         if self.kwargs:
-            sig = sig + ("**%s, " % self.kwargs)
+            sig.append("**" + self.kwargs)
 
-        # slice off the last comma and space
-        if self.positional or self.varargs or self.kwargs:
-            sig = sig[:-2]
-
-        sig = sig + ")"
-        return sig
+        return "(%s)" % ", ".join(sig)
 
 
-def fromFunction(func, interface='', imlevel=0, name=None):
+def fromFunction(func, interface=None, imlevel=0, name=None):
     name = name or func.__name__
-    m=Method(name, func.__doc__)
-    defaults=func.func_defaults or ()
-    c=func.func_code
-    na=c.co_argcount-imlevel
-    names=c.co_varnames[imlevel:]
-    d={}
-    nr=na-len(defaults)
+    method = Method(name, func.__doc__)
+    defaults = func.func_defaults or ()
+    code = func.func_code
+    # Number of positional arguments
+    na = code.co_argcount-imlevel
+    names = code.co_varnames[imlevel:]
+    opt = {}
+    # Number of required arguments
+    nr = na-len(defaults)
     if nr < 0:
         defaults=defaults[-nr:]
-        nr=0
+        nr = 0
 
-    for i in range(len(defaults)):
-        d[names[i+nr]]=defaults[i]
+    # Determine the optional arguments.
+    opt.update(dict(zip(names[nr:], defaults)))
 
-    m.positional=names[:na]
-    m.required=names[:nr]
-    m.optional=d
+    method.positional = names[:na]
+    method.required = names[:nr]
+    method.optional = opt
 
     argno = na
-    if c.co_flags & CO_VARARGS:
-        m.varargs = names[argno]
+
+    # Determine the function's variable argument's name (i.e. *args)
+    if code.co_flags & CO_VARARGS:
+        method.varargs = names[argno]
         argno = argno + 1
     else:
-        m.varargs = None
-    if c.co_flags & CO_VARKEYWORDS:
-        m.kwargs = names[argno]
+        method.varargs = None
+
+    # Determine the function's keyword argument's name (i.e. **kw)
+    if code.co_flags & CO_VARKEYWORDS:
+        method.kwargs = names[argno]
     else:
-        m.kwargs = None
+        method.kwargs = None
 
-    m.interface=interface
+    method.interface = interface
 
-    for k, v in func.__dict__.items():
-        m.setTaggedValue(k, v)
+    for key, value in func.__dict__.items():
+        method.setTaggedValue(key, value)
 
-    return m
+    return method
 
-def fromMethod(meth, interface=''):
+
+def fromMethod(meth, interface=None, name=None):
     func = meth.im_func
-    return fromFunction(func, interface, imlevel=1)
+    return fromFunction(func, interface, imlevel=1, name=name)
 
 
 # Now we can create the interesting interfaces and wire them up:
@@ -908,8 +801,9 @@ def _wire():
     from zope.interface.interfaces import IMethod
     classImplements(Method, IMethod)
 
-    from zope.interface.interfaces import IInterface
+    from zope.interface.interfaces import IInterface, ISpecification
     classImplements(InterfaceClass, IInterface)
+    classImplements(Specification, ISpecification)
 
 # We import this here to deal with module dependencies.
 from zope.interface.declarations import providedBy, implementedBy
